import logging
import os
import subprocess

def dbg_check_call(*args, **kwargs):
    logger = logging.getLogger(__name__)
    logger.debug(f"Running: {' '.join(*args)}")
    subprocess.check_call(*args, **kwargs)

def delete_tempfile():
    return os.environ.get('DEB_EDK2_DEBUG', 'false') in ['0', 'false']
